# -*- coding: utf-8 -*-
"""test subscription"""

from __future__ import unicode_literals

import os
from datetime import date

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import mail
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from django.utils.translation import activate

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm import models
from balafon.Crm.signals import new_subscription


@override_settings(BALAFON_SUBSCRIBE_ENABLED=True)
class SubscribeTest(TestCase):
    """Subscribe to newsletter"""

    @staticmethod
    def email_as_string(email):
        return email.message().as_string()

    def setUp(self):
        """before each test"""
        super(SubscribeTest, self).setUp()
        self._lang = settings.LANGUAGES[0][0]
        activate(self._lang)

        os.environ['RECAPTCHA_DISABLE'] = 'True'

        if not getattr(settings, 'BALAFON_ALLOW_SINGLE_CONTACT', True):
            settings.BALAFON_INDIVIDUAL_ENTITY_ID = models.EntityType.objects.create(name="particulier").id

        mommy.make(models.Zone, name=settings.BALAFON_DEFAULT_COUNTRY, parent=None)

    def tearDown(self):
        """after each test"""
        activate(self._lang)
        os.environ['RECAPTCHA_DISABLE'] = 'False'
        super(SubscribeTest, self).tearDown()

    def test_view_subscribe_newsletter(self):
        """view subscription page"""
        url = reverse("emailing_subscribe_newsletter")
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_subscribe_newsletter_no_email(self):
        """subscribe without setting an email"""
        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)

        url = reverse("emailing_subscribe_newsletter")

        data = {
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'groups': str(group1.id),
            'entity_type': 0,
            'email': '',
        }
        self._patch_with_captcha(url, data)

        self.assertEqual(models.Contact.objects.count(), 0)
        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)
        self.assertEqual(models.Contact.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_subscribe_newsletter_message(self):
        """subscribe with a message"""
        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': "pierre.dupond@mon-mail.fr",
            'message': "Hello",
        }
        self._patch_with_captcha(url, data)

        self.assertEqual(models.Contact.objects.count(), 0)
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)
        contact = models.Contact.objects.all()[0]

        self.assertEqual(contact.email, data["email"])
        self.assertEqual(contact.lastname, data["lastname"])
        self.assertEqual(contact.firstname, data["firstname"])

        self.assertEqual(1, contact.action_set.count())
        action = contact.action_set.all()[0]
        self.assertEqual(data["message"], action.detail)

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])

    def test_subscribe_newsletter_empty_message(self):
        """subscribe with an empty message"""
        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': "pierre.dupond@mon-mail.fr",
            'message': "",
        }
        self._patch_with_captcha(url, data)

        self.assertEqual(models.Contact.objects.count(), 0)
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(200, response.status_code)

        self.assertEqual(models.Contact.objects.count(), 1)
        contact = models.Contact.objects.all()[0]

        self.assertEqual(contact.email, data["email"])
        self.assertEqual(contact.lastname, data["lastname"])
        self.assertEqual(contact.firstname, data["firstname"])

        self.assertEqual(0, contact.action_set.count())

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])

    def test_subscribe_newsletter_no_entity(self):
        """subscribe as an individual"""
        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)

        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'groups': str(group1.id),
            'email': 'pdupond@apidev.fr',
        }
        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertEqual(contact.favorite_language, '')
        self.assertEqual(contact.get_same_as().count(), 0)
        self.assertEqual(contact.get_same_email().count(), 0)

        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_done', args=[contact.uuid])) >= 0)

        self.assertEqual(contact.lastname, data['lastname'])
        self.assertEqual(contact.firstname, data['firstname'])
        self.assertEqual(list(contact.entity.group_set.all()), [group1])

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])

    @override_settings(LANGUAGES=(('en', 'English'), ('fr', 'French')))
    def test_subscribe_newsletter_favorite_language(self):
        """subscribe as an individual"""
        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'groups': [],
            'email': 'pdupond@apidev.fr',
            'favorite_language': 'fr',
        }
        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertEqual(contact.favorite_language, 'fr')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_done', args=[contact.uuid])) >= 0)

        self.assertEqual(contact.lastname, data['lastname'])
        self.assertEqual(contact.firstname, data['firstname'])

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])

    def test_subscribe_existing_contact(self):
        """subscribe again to mailing list"""
        entity = mommy.make(models.Entity)
        existing_contact = entity.default_contact
        existing_contact.email = 'pdupond@apidev.fr'
        existing_contact.save()

        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'groups': [],
            'email': existing_contact.email,
        }
        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 2)
        self.assertEqual(models.Contact.objects.filter(email=existing_contact.email).count(), 2)

        new_contact = models.Contact.objects.filter(email=existing_contact.email).exclude(id=existing_contact.id)[0]
        self.assertEqual(new_contact.email_verified, False)

        self.assertEqual(new_contact.get_same_as().count(), 0)
        self.assertEqual(new_contact.get_same_email().count(), 1)

        self.assertNotEqual(new_contact.uuid, '')
        self.assertEqual(new_contact.favorite_language, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_done', args=[new_contact.uuid])) >= 0)

        self.assertEqual(new_contact.lastname, data['lastname'])
        self.assertEqual(new_contact.firstname, data['firstname'])

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [new_contact.email])
        url = reverse('emailing_email_verification', args=[new_contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        email_content = self.email_as_string(notification_email)
        self.assertTrue(email_content.find(new_contact.fullname) > 0)
        self.assertTrue(email_content.find(new_contact.get_absolute_url()) > 0)
        self.assertTrue(email_content.find(existing_contact.get_absolute_url()) > 0)

    def test_subscribe_several_existing_contact(self):
        """subscribe again to mailing list"""
        entity1 = mommy.make(models.Entity)
        existing_contact1 = entity1.default_contact
        existing_contact1.email = 'pdupond@apidev.fr'
        existing_contact1.save()

        entity2 = mommy.make(models.Entity, email=existing_contact1.email)
        existing_contact2 = entity2.default_contact
        existing_contact4 = mommy.make(models.Contact, entity=entity2)

        entity3 = mommy.make(models.Entity)
        existing_contact3 = entity3.default_contact
        existing_contact3.email = existing_contact1.email
        existing_contact3.save()

        mommy.make(models.Contact, email="toto@somethingelse.fr")

        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'groups': [],
            'email': existing_contact1.email,
        }
        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.filter(email=existing_contact1.email).count(), 3)

        existing_ids = [
            contact.id for contact in (existing_contact1, existing_contact2, existing_contact3, existing_contact4)
        ]
        queryset = models.Contact.objects.filter(email=existing_contact1.email).exclude(id__in=existing_ids)
        self.assertEqual(queryset.count(), 1)
        new_contact = queryset[0]
        self.assertEqual(new_contact.email_verified, False)

        self.assertEqual(new_contact.get_same_as().count(), 0)
        self.assertEqual(new_contact.get_same_email().count(), 4)
        self.assertEqual(
            sorted([contact.id for contact in new_contact.get_same_email()]),
            sorted(existing_ids)
        )

        self.assertNotEqual(new_contact.uuid, '')
        self.assertEqual(new_contact.favorite_language, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_done', args=[new_contact.uuid])) >= 0)

        self.assertEqual(new_contact.lastname, data['lastname'])
        self.assertEqual(new_contact.firstname, data['firstname'])

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [new_contact.email])
        url = reverse('emailing_email_verification', args=[new_contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        email_content = self.email_as_string(notification_email)
        self.assertTrue(email_content.find(new_contact.fullname) > 0)
        self.assertTrue(email_content.find(new_contact.get_absolute_url()) > 0)
        for existing_contact in (existing_contact1, existing_contact2, existing_contact3, existing_contact4):
            self.assertTrue(email_content.find(existing_contact.get_absolute_url()) > 0)

    def test_subscribe_newsletter_entity(self):
        """subscribe as a company member"""
        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
        group2 = mommy.make(models.Group, name="DEF", subscribe_form=True)

        url = reverse("emailing_subscribe_newsletter")

        entity_type = mommy.make(models.EntityType, name='Pro', subscribe_form=True)

        data = {
            'entity_type': entity_type.id,
            'entity': 'Toto',
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': 'pdupond@apidev.fr',
            'groups': [group1.id, group2.id],
        }
        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)

        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_done', args=[contact.uuid])) >= 0)

        self.assertEqual(contact.entity.name, data['entity'])
        self.assertEqual(contact.lastname, data['lastname'])
        self.assertEqual(contact.firstname, data['firstname'])
        self.assertEqual(list(contact.entity.group_set.all()), [group1, group2])

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])

    def test_subscribe_newsletter_private_group(self):
        """test we can not subscribe to a private group"""
        group1 = mommy.make(models.Group, name="ABC", subscribe_form=True)
        group2 = mommy.make(models.Group, name="DEF", subscribe_form=False)

        url = reverse("emailing_subscribe_newsletter")

        entity_type = mommy.make(models.EntityType, name='Pro', subscribe_form=True)
        data = {
            'entity_type': entity_type.id,
            'entity': 'Toto',
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': 'pdupond@apidev.fr',
            'groups': [group1.id, group2.id],
        }
        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Entity.objects.count(), 0)
        self.assertEqual(models.Contact.objects.count(), 0)

        self.assertEqual(len(mail.outbox), 0)

    def test_view_subscribe_done(self):
        """test subscribe done page"""
        contact = mommy.make(models.Contact)
        url = reverse('emailing_subscribe_done', args=[contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def _patch_with_captcha(self, url, data):
        """path form data with captcha"""
        pass

    def test_view_subscribe_with_types(self):
        """view subscribe page"""
        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)

        subscribe_type1 = mommy.make(models.SubscriptionType, name="#News#abc", site=site1)
        subscribe_type2 = mommy.make(models.SubscriptionType, name="#News#def", site=site1)
        subscribe_type3 = mommy.make(models.SubscriptionType, name="#News#ghi", site=site2)
        subscribe_type4 = mommy.make(models.SubscriptionType, name="#News#jkl")

        url = reverse("emailing_subscribe_newsletter")
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, subscribe_type1.name)
        self.assertContains(response, subscribe_type2.name)
        self.assertNotContains(response, subscribe_type3.name)
        self.assertNotContains(response, subscribe_type4.name)

    @override_settings(LANGUAGES=(('en', 'English'), ('fr', 'French')))
    def test_view_subscription_language(self):
        """make sure the favorite_language is set correctly"""
        activate('fr')

        mommy.make(models.SubscriptionType)

        url = reverse("emailing_subscribe_newsletter")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        self.assertEqual(1, len(soup.select("#id_favorite_language")))
        self.assertEqual('fr', soup.select("#id_favorite_language")[0]["value"])

    @override_settings(LANGUAGES=(('en', 'English'),))
    def test_view_subscription_no_language(self):
        """make sure the favorite_language is set correctly"""
        mommy.make(models.SubscriptionType)

        url = reverse("emailing_subscribe_newsletter")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        self.assertEqual(1, len(soup.select("#id_favorite_language")))
        self.assertEqual('', soup.select("#id_favorite_language")[0].get("value", ""))

    def test_accept_newsletter_not_in_site(self):
        """if subscribtion is not set on the site, it should not be possible to subscribe"""
        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=None)

        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': 'pdupond@apidev.fr',
            'subscription_types': [newsletter_subscription.id],
        }

        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 0)
        self.assertEqual(models.Subscription.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    def test_accept_newsletter(self, accept_newsletter=True, accept_3rdparty=True):
        """subscribe and accept some subscribtions"""
        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)
        third_party_subscription = mommy.make(models.SubscriptionType, name="3rd_party", site=site1)

        url = reverse("emailing_subscribe_newsletter")

        subscription_types = []
        if accept_newsletter:
            subscription_types.append(newsletter_subscription.id)
        if accept_3rdparty:
            subscription_types.append(third_party_subscription.id)

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': 'pdupond@apidev.fr',
            'subscription_types': subscription_types,
        }

        self._patch_with_captcha(url, data)

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_done', args=[contact.uuid])) >= 0)

        self.assertEqual(contact.lastname, data['lastname'])
        self.assertEqual(contact.firstname, data['firstname'])

        subscription_newsletter = models.Subscription.objects.get(
            contact=contact, subscription_type=newsletter_subscription
        )
        self.assertEqual(subscription_newsletter.accept_subscription, accept_newsletter)
        if accept_newsletter:
            self.assertEqual(subscription_newsletter.subscription_date.date(), date.today())
        else:
            self.assertEqual(subscription_newsletter.subscription_date, None)

        subscription_3rdparty = models.Subscription.objects.get(
            contact=contact, subscription_type=third_party_subscription
        )
        self.assertEqual(subscription_3rdparty.accept_subscription, accept_3rdparty)
        if accept_3rdparty:
            self.assertEqual(subscription_3rdparty.subscription_date.date(), date.today())
        else:
            self.assertEqual(subscription_3rdparty.subscription_date, None)

        self.assertEqual(contact.email_verified, False)

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_string(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])

    def test_refuse_newsletter(self):
        """test refuse all newsletters"""
        self.test_accept_newsletter(accept_newsletter=False, accept_3rdparty=False)

    def test_refuse_newsletter_accept_3rdparty(self):
        """test accept only third parties"""
        self.test_accept_newsletter(accept_newsletter=False, accept_3rdparty=True)

    def test_accept_newsletter_refuse_3rdparty(self):
        """test accept only newsletter"""
        self.test_accept_newsletter(accept_newsletter=True, accept_3rdparty=False)

    def test_verify_email(self):
        """test email verification"""
        self.client.logout()

        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)
        third_party_subscription = mommy.make(models.SubscriptionType, name="3rd_party", site=site1)
        contact = mommy.make(models.Contact, email='toto@apidev.fr', email_verified=False)

        subscription1 = mommy.make(
            models.Subscription, contact=contact, subscription_type=newsletter_subscription, accept_subscription=True
        )
        subscription2 = mommy.make(
            models.Subscription, contact=contact, subscription_type=third_party_subscription, accept_subscription=True
        )

        url = reverse('emailing_email_verification', args=[contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        contact = models.Contact.objects.get(id=contact.id)
        self.assertEqual(contact.email_verified, True)

        subscription1 = models.Subscription.objects.get(id=subscription1.id)
        self.assertEqual(subscription1.accept_subscription, True)
        subscription2 = models.Subscription.objects.get(id=subscription2.id)
        self.assertEqual(subscription2.accept_subscription, True)

    def test_verify_email_no_newsletter(self):
        """test email verification ofr some who refuse subscriptions"""
        self.client.logout()

        site1 = Site.objects.get_current()

        newsletter_subscription = mommy.make(models.SubscriptionType, name="newsletter", site=site1)
        third_party_subscription = mommy.make(models.SubscriptionType, name="3rd_party", site=site1)
        contact = mommy.make(models.Contact, email='toto@apidev.fr', email_verified=False)

        subscription1 = mommy.make(
            models.Subscription, contact=contact, subscription_type=newsletter_subscription, accept_subscription=False
        )
        subscription2 = mommy.make(
            models.Subscription, contact=contact, subscription_type=third_party_subscription, accept_subscription=False
        )

        url = reverse('emailing_email_verification', args=[contact.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        contact = models.Contact.objects.get(id=contact.id)
        self.assertEqual(contact.email_verified, True)
        subscription1 = models.Subscription.objects.get(id=subscription1.id)
        self.assertEqual(subscription1.accept_subscription, False)
        subscription2 = models.Subscription.objects.get(id=subscription2.id)
        self.assertEqual(subscription2.accept_subscription, False)

    def test_verify_email_strange_uuid(self):
        """test accept verification with an invalid uuid should fail"""
        self.client.logout()
        contact = mommy.make(models.Contact, email='toto@apidev.fr', email_verified=False)

        url = reverse('emailing_email_verification', args=['abcd'])
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)
        contact = models.Contact.objects.get(id=contact.id)
        self.assertEqual(contact.email_verified, False)

    @override_settings(BALAFON_SUBSCRIBE_ENABLED=False)
    def test_view_subscribe_disabled(self):
        url = reverse("emailing_subscribe_newsletter")

        response = self.client.get(url,)
        self.assertEqual(404, response.status_code)

        self.assertEqual(models.Contact.objects.count(), 0)

    @override_settings(BALAFON_SUBSCRIBE_ENABLED=False)
    def test_subscribe_disabled(self):
        url = reverse("emailing_subscribe_newsletter")

        data = {
            'entity_type': 0,
            'lastname': 'Dupond',
            'firstname': 'Pierre',
            'email': "pierre.dupond@mon-mail.fr",
            'message': "",
        }

        self.assertEqual(models.Contact.objects.count(), 0)
        response = self.client.post(url, data=data, follow=True)
        self.assertEqual(404, response.status_code)

        self.assertEqual(models.Contact.objects.count(), 0)

        # email verification
        self.assertEqual(len(mail.outbox), 0)

    def test_accept_newsletter_signal(self):
        """subscribe and accept some subscribtions"""

        def add_to_group_callback(sender, instance, contact, **kwargs):
            group = models.Group.objects.get_or_create(name=instance.subscription_type.name)[0]
            group.contacts.add(contact)
            group.save()

        new_subscription.connect(add_to_group_callback, sender=models.Subscription)

        self.test_accept_newsletter(accept_newsletter=True, accept_3rdparty=False)

        self.assertEqual(1, models.Contact.objects.count())
        contact = models.Contact.objects.all()[0]

        # Check the callbacks have been called
        self.assertEqual(models.Group.objects.count(), 1)
        group = models.Group.objects.get(name='newsletter')
        self.assertEqual(group.entities.count(), 0)
        self.assertEqual(list(group.contacts.all()), [contact])

        new_subscription.disconnect(add_to_group_callback, sender=models.Subscription)



# -*- coding: utf-8 -*-
"""REST api powered by django-rest-framework"""

from __future__ import unicode_literals

from decimal import Decimal

from django.db.models import Q
from django.core.exceptions import PermissionDenied
from django.utils.translation import ugettext as _

from rest_framework import viewsets, permissions
from rest_framework.response import Response
from rest_framework.views import APIView

from balafon.settings import is_profile_installed
from balafon.Crm.models import Action, ActionType
from balafon.Profile.models import ContactProfile
from balafon.Store.models import (
    Favorite, SaleItem, StoreItem, StoreItemCategory, StoreItemTag, StoreManagementActionType, DeliveryPoint,
    SaleAnalysisCode
)
from balafon.Store import settings
from balafon.Store.api import serializers
from balafon.Store.settings import get_cart_type_name, get_cart_processed_callback
from balafon.Store.utils import notify_cart_to_admin, confirm_cart_to_user, logger


class CanAccessStorePermission(permissions.BasePermission):
    """Define who can access the store"""

    def has_permission(self, request, view):
        """check permission"""

        # Allow authenticated users
        if request.user.is_authenticated():
            return True

        # Allow anonymous user if settings
        return settings.is_public_api_allowed()


def get_public_api_permissions():
    """get public api permissions"""
    return [CanAccessStorePermission]


class SaleItemViewSet(viewsets.ModelViewSet):
    """returns sales items"""
    queryset = SaleItem.objects.all()
    serializer_class = serializers.SaleItemSerializer
    permission_classes = [permissions.IsAuthenticated]

    def get_serializer_class(self, *args, **kwargs):
        if self.request.method in ('PUT', 'POST'):
            return serializers.UpdateSaleItemSerializer
        else:
            return super(SaleItemViewSet, self).get_serializer_class(*args, **kwargs)

    def get_queryset(self):
        """returns objects"""
        action_id = self.kwargs['action_id']
        return self.queryset.filter(sale__action__id=action_id)


class StoreItemViewSet(viewsets.ModelViewSet):
    """returns sales items"""
    queryset = StoreItem.objects.filter(published=True)
    serializer_class = serializers.StoreItemSerializer
    permission_classes = get_public_api_permissions()

    def get_queryset(self):
        """returns objects"""
        contact = None
        if is_profile_installed():
            try:
                if self.request.user.is_authenticated():
                    profile = ContactProfile.objects.get(user=self.request.user)
                    contact = profile.contact
            except ContactProfile.DoesNotExist:
                pass

        if contact:
            # The contact exists : show items with a only_for_groups if contact is member of one of the groups
            self.queryset = self.queryset.filter(
                Q(only_for_groups=None) |
                Q(only_for_groups__contacts=contact) |
                Q(only_for_groups__entities=contact.entity)
            ).distinct()
        else:
            # The contact doesn't exist : Don't show items with a only_for_groups
            self.queryset = self.queryset.filter(only_for_groups=None)

        name = self.request.GET.get('name', None)
        if name:
            return self.queryset.filter(name__icontains=name)[:20]

        fullname = self.request.GET.get('fullname', None)
        if fullname:
            return self.queryset.filter(name__icontains=fullname)

        category = self.request.GET.get('category', None)
        if category:
            return self.queryset.filter(category=category)

        tag = self.request.GET.get('tag', None)
        if tag:
            return self.queryset.filter(tags=tag)

        ids = self.request.GET.get('ids', None)
        if ids is not None:
            int_ids = [int(id_) for id_ in ids.split(',') if id_.isdigit()]
            return self.queryset.filter(id__in=int_ids)

        return self.queryset.prefetch_related("tags")


class StoreItemCategoryViewSet(viewsets.ModelViewSet):
    """returns categories"""
    queryset = StoreItemCategory.objects.all()
    serializer_class = serializers.StoreItemCategorySerializer
    permission_classes = get_public_api_permissions()


class StoreItemTagViewSet(viewsets.ModelViewSet):
    """returns tags"""
    queryset = StoreItemTag.objects.all()
    serializer_class = serializers.StoreItemTagSerializer
    permission_classes = get_public_api_permissions()


class CartView(APIView):
    """post a cart"""
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request):
        """receive a new cart"""

        cart_serializer = serializers.CartSerializer(data=request.data)
        if cart_serializer.is_valid():

            # Get Contact
            try:
                profile = ContactProfile.objects.get(user=request.user)
                contact = profile.contact
            except ContactProfile.DoesNotExist:
                return Response({'ok': False, 'message': _("You don't have a valid profile")})

            # Get Delivery point
            try:
                delivery_point = DeliveryPoint.objects.get(id=cart_serializer.validated_data["delivery_point"])
            except DeliveryPoint.DoesNotExist:
                return Response({'ok': False, 'message': _("Invalid delivery point")})

            # Create a new Sale
            action_type_name = get_cart_type_name()
            action_type = ActionType.objects.get_or_create(name=action_type_name)[0]

            # Force this type to be managed by the store
            StoreManagementActionType.objects.get_or_create(action_type=action_type)

            subject = detail = ''
            if 'notes' in cart_serializer.validated_data:
                notes = cart_serializer.validated_data["notes"].strip()
                if notes:
                    subject = _('Notes')
                    detail = notes

            action = Action.objects.create(
                planned_date=cart_serializer.validated_data['purchase_datetime'],
                subject=subject,
                detail=detail,
                status=action_type.default_status,
                status2=action_type.default_status2,
                type=action_type
            )
            action.contacts.add(contact)
            action.save()

            action.sale.analysis_code = SaleAnalysisCode.objects.get_or_create(name="Internet")[0]

            action.sale.delivery_point = delivery_point
            action.sale.save()

            warnings = []
            is_empty = True
            # for each line add a sale item
            for index, item in enumerate(cart_serializer.validated_data['items']):

                quantity = Decimal('{0}'.format(item['quantity']))

                try:
                    store_item = StoreItem.objects.get(id=item['id'])
                except StoreItem.DoesNotExist:
                    # ignore if not existing
                    continue

                if not store_item.available:
                    warnings.append(
                        _("{0} is currently not available. It has been removed from your cart").format(
                            store_item.name
                        )
                    )
                else:
                    if store_item.stock_count is not None:
                        store_item.stock_count = store_item.stock_count - quantity
                        store_item.save()

                    is_empty = False
                    SaleItem.objects.create(
                        sale=action.sale,
                        quantity=quantity,
                        item=store_item,
                        vat_rate=store_item.vat_rate,
                        pre_tax_price=store_item.pre_tax_price,
                        text=store_item.name,
                        order_index=index + 1
                    )

            # Done
            if is_empty:
                action.delete()

                return Response({
                    'ok': False,
                    'message': _('Your cart is empty. Your articles are not available'),
                    'warnings': warnings,
                })
            else:

                action.sale.save()

                on_cart_processed = get_cart_processed_callback()
                if on_cart_processed:
                    on_cart_processed(action)

                confirm_cart_to_user(profile, action)
                notify_cart_to_admin(profile, action)

                return Response({
                    'ok': True,
                    'warnings': warnings,
                    'deliveryDate': action.planned_date,
                    'deliveryPlace': action.sale.delivery_point.name,
                })
        else:
            return Response(
                {
                    'ok': False,
                    'message': ', '.join(['{0}: {1}'.format(*err) for err in cart_serializer.errors.items()])
                }
            )


class ProfileAPIView(APIView):
    """base class"""

    def get_profile(self, request):
        """return a valid profile"""
        # Get Contact
        try:
            profile = ContactProfile.objects.get(user=request.user)
            if not profile.contact:
                raise PermissionDenied
            return profile

        except ContactProfile.DoesNotExist:
            raise PermissionDenied


class FavoriteView(ProfileAPIView):
    """manage favorite store items"""
    permission_classes = (permissions.IsAuthenticated,)

    def post(self, request):
        """receive a new list of favorites"""

        favorite_serializer = serializers.FavoriteSerializer(data=request.data)
        if favorite_serializer.is_valid():

            # Check that the user has a valid profile
            self.get_profile(request)

            items_ids = [item_['id'] for item_ in favorite_serializer.validated_data['items']]

            # Delete favorites which are not in the new list
            request.user.favorite_set.exclude(id__in=items_ids).delete()

            # create new favorite
            for item_id in items_ids:
                try:
                    item = StoreItem.objects.get(id=item_id)
                except StoreItem.DoesNotExist:
                    item = None
                if item:
                    Favorite.objects.get_or_create(
                        user=request.user,
                        item=item
                    )

            return Response({'ok': True})
        else:
            return Response(
                {
                    'ok': False,
                    'message': ', '.join(['{0}: {1}'.format(*err) for err in favorite_serializer.errors.items()])
                }
            )

    def get(self, request):
        """return a new list of favorites"""

        # Check that the user has a valid profile
        self.get_profile(request)

        favorites = self.request.user.favorite_set.all()
        favorites_serializer = serializers.FavoriteItemSerializer(
            [fav.item for fav in favorites if fav.item.published],
            many=True
        )
        return Response({'favorites': favorites_serializer.data})


class LastSalesView(ProfileAPIView):
    """view last sales"""
    permission_classes = (permissions.IsAuthenticated,)

    def get(self, request):
        """return a last items bought by the user"""

        # Check that the user has a valid profile
        profile = self.get_profile(request)

        sale_items = SaleItem.objects.filter(
            sale__action__contacts=profile.contact
        ).order_by('-sale__action__planned_date', 'item__name')

        # Avoid duplicates
        already_in = set()
        store_items = []
        for sale_item in sale_items:
            if sale_item.item and sale_item.item.published and sale_item.item.id not in already_in:
                already_in.add(sale_item.item.id)
                store_items.append(sale_item.item)

        last_sales_serializer = serializers.StoreItemSerializer(store_items, many=True)

        return Response(last_sales_serializer.data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0010_clean_same_as'),
        ('Store', '0003_auto_20160520_1602'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleanalysiscode',
            name='action_type',
            field=models.ForeignKey(default=None, blank=True, to='Crm.ActionType', null=True, verbose_name='action type'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.core.mail import EmailMessage
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from balafon.utils import logger, now_rounded
from balafon.Crm import settings as crm_settings
from balafon.Crm.models import Contact, Entity, EntityType, Action, ActionType, Group
from balafon.Emailing.models import SubscriptionType
from balafon.Emailing.utils import save_subscriptions
from balafon.Profile.models import ContactProfile, CategoryPermission


def create_profile_contact(user):
    profile = user.contactprofile
    if not profile:
        profile = ContactProfile(user=user)
    rename_entity = False
    
    if profile.contact:
        contact = profile.contact
    else:
        warn_duplicates = False
        contact = None
        try:
            contact = Contact.objects.get(email=user.email)
        except Contact.DoesNotExist:
            pass
        except Contact.MultipleObjectsReturned:
            warn_duplicates = True
        
        if not contact:
            if profile and profile.entity_type:
                entity_type = profile.entity_type
            else:
                if crm_settings.ALLOW_SINGLE_CONTACT:
                    entity_type = None
                else:
                    et_id = getattr(settings, 'BALAFON_INDIVIDUAL_ENTITY_ID', 1)
                    entity_type, x = EntityType.objects.get_or_create(id=et_id)
                    rename_entity = True

            entity = Entity(
                name=profile.entity_name if profile else user.username,
                is_single_contact=(entity_type is None) and crm_settings.ALLOW_SINGLE_CONTACT,
                type=entity_type
            )
            
            entity.save()
            # This create a default contact
            contact = entity.default_contact

        if warn_duplicates:
            action_type = ActionType.objects.get_or_create(name=_("Balafon admin"))[0]
            action = Action.objects.create(
                subject=_("A user have registred with email {0} used by several other contacts".format(user.email)),
                type=action_type,
                planned_date=now_rounded(),
                detail=_('You should check that this contact is not duplicated'),
                display_on_board=True
            )
            action.contacts.add(contact)
            action.save()

    fields = (
        'gender', 'firstname', 'lastname', 'phone', 'mobile', 'address', 'address2', 'address3',
        'zip_code', 'city', 'cedex', 'country',
    )
    if profile:
        for field in fields:
            value = getattr(profile, field, None)
            if value:
                setattr(contact, field, value)

    contact.lastname = contact.lastname or user.last_name
    contact.firstname = contact.firstname or user.first_name
    contact.email = user.email
    contact.email_verified = True
    if not contact.lastname:
        contact.lastname = user.email.split("@")[0]

    contact.save()

    try:
        ids = [int(s) for s in profile.subscriptions_ids.split(",")]
    except ValueError:
        ids = []

    subscription_types = []
    for subscription_type_id in ids:
        try:
            subscription_types.append(SubscriptionType.objects.get(id=subscription_type_id))
        except SubscriptionType.DoesNotExist:
            pass
    save_subscriptions(contact, subscription_types)

    try:
        groups_ids = [int(s) for s in profile.groups_ids.split(",")]
    except ValueError:
        groups_ids = []

    for group_id in groups_ids:
        try:
            group = Group.objects.get(id=group_id)
            group.contacts.add(contact)
            group.save()
        except Group.DoesNotExist:
            pass

    action_type = ActionType.objects.get_or_create(name=_("Account creation"))[0]
    action = Action.objects.create(
        subject=_("Create an account on web site"),
        type=action_type,
        planned_date=now_rounded(),
        display_on_board=False,
        done=True
    )
    action.contacts.add(contact)
    action.save()
    
    if rename_entity:
        contact.entity.name = "{0.lastname} {0.firstname}".format(contact).strip().upper()
        contact.entity.save()
    
    profile.contact = contact
    profile.save()
    return profile


def check_category_permission(obj, permission, user):
    """check that the object has a category"""
    
    try:
        cat_perm = CategoryPermission.objects.get(category=obj.category)
    except AttributeError:
        # logger.debug('object category has no category')
        return True
    except CategoryPermission.DoesNotExist:
        # If no category permission exists : anyone is allowed
        # logger.debug('object category has no permission defined')
        return True
    
    # Get the contact corresponding to the logged user
    try:
        contact = user.contactprofile.contact
        if not contact.id:
            return False
    except (ContactProfile.DoesNotExist, Contact.DoesNotExist, AttributeError):
        # If anonymous user or no contact exists for this profile
        # users are not allowed to check the category
        # logger.debug("user contact does't exist")
        return False
    
    if permission == 'can_view_article':
        groups = cat_perm.can_view_groups
    elif permission == 'can_edit_article':
        groups = cat_perm.can_edit_groups
    elif permission == "can_download_file":
        groups = cat_perm.can_view_groups
    else:
        #This perm is not managed : allow it
        return True
    
    for group in groups.all():
        if group.contacts.filter(id=contact.id).count() or group.entities.filter(id=contact.entity.id).count():
            # user is member of an allowed group
            return True
    # user is not member of a group : do not allow
    return False


def notify_registration(profile):
    """send message by email"""
    notification_email = getattr(settings, 'BALAFON_NOTIFICATION_EMAIL', '')
    if notification_email:
        from_email = getattr(settings, 'DEFAULT_FROM_EMAIL')
        
        data = {
            'contact': profile.contact,
            'site': getattr(settings, 'COOP_CMS_SITE_PREFIX', None),
        }
        the_template = get_template('Profile/registration_notification_email.txt')
        content = the_template.render(data)
        email = EmailMessage(
            _("New registration"), content, from_email,
            [notification_email], headers={'Reply-To': profile.contact.email})
        try:
            email.send()
        except Exception:
            logger.exception("notify_registration")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0012_auto_20160601_1128'),
    ]

    operations = [
        migrations.CreateModel(
            name='MailtoSettings',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('bcc', models.BooleanField(default=False, verbose_name='carbon copy')),
                ('subject', models.CharField(default=b'', help_text='Use action subject if empty', max_length=100, verbose_name='subject', blank=True)),
                ('body_template', models.TextField(default=b'', verbose_name='body template', blank=True)),
                ('action_type', models.OneToOneField(verbose_name='action type', to='Crm.ActionType')),
            ],
            options={
                'verbose_name': 'Mailto settings',
                'verbose_name_plural': 'Mailto settings',
            },
        ),
    ]



# -*- coding: utf-8 -*-
"""context processor"""

from __future__ import unicode_literals

from datetime import datetime

from django.conf import settings
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db.models import Q

from balafon import VERSION
from balafon.Crm.models import EntityType, ActionType
from balafon.Crm import settings as crm_settings
from balafon.utils import is_allowed_homepage


def crm(request):
    """add constant to context"""
    
    site_url = None
    if not settings.BALAFON_AS_HOMEPAGE:
        try:
            site_url = reverse('homepage')
        except NoReverseMatch:
            pass

    return {
        'BALAFON_VERSION': VERSION,
        'BALAFON_SITE_URL': site_url,
        'BALAFON_ALLOW_SINGLE_CONTACT': crm_settings.ALLOW_SINGLE_CONTACT,
        'BALAFON_NO_ENTITY_TYPE': crm_settings.NO_ENTITY_TYPE,
        'BALAFON_ENTITY_TYPES': EntityType.objects.all(),
        'BALAFON_MULTI_USER': getattr(settings, 'BALAFON_MULTI_USER', True),
        'BALAFON_EMAIL_LOGIN': ('balafon.Profile.backends.EmailModelBackend' in settings.AUTHENTICATION_BACKENDS),
        'BALAFON_STORE_INSTALLED': 'balafon.Store' in settings.INSTALLED_APPS,
        'NOW': datetime.now(),
        'is_allowed_homepage': is_allowed_homepage(request.path),
        'addable_action_types': ActionType.objects.filter(Q(set__isnull=False) | Q(is_default=True)),
        'has_default_action': ActionType.objects.filter(is_default=True).count(),
    }



# -*- coding: utf-8 -*-
"""models"""

from __future__ import unicode_literals

from datetime import datetime
import uuid
import unicodedata

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericRelation
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models import signals
from django.utils.dateformat import DateFormat
from django.utils.encoding import python_2_unicode_compatible
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext, ugettext_lazy as _

from coop_cms.models import Newsletter
from django_extensions.db.models import TimeStampedModel

from balafon.Crm.models import Contact, Action, SubscriptionType
from balafon.Crm.settings import get_language_choices
from balafon.Users.models import UserPreferences, Favorite


@python_2_unicode_compatible
class Emailing(TimeStampedModel):
    """configuration on an emailing"""
    
    STATUS_EDITING = 1
    STATUS_SCHEDULED = 2
    STATUS_SENDING = 3
    STATUS_SENT = 4
    
    STATUS_CHOICES = (
        (STATUS_EDITING, _('Edition in progress')),
        (STATUS_SCHEDULED, _('Sending is scheduled')),
        (STATUS_SENDING, _('Sending in progress')),
        (STATUS_SENT, _('Sent')),
    )

    class Meta:
        verbose_name = _('Emailing')
        verbose_name_plural = _('Emailings')

    subscription_type = models.ForeignKey(SubscriptionType)
    newsletter = models.ForeignKey(Newsletter) 
    send_to = models.ManyToManyField(Contact, blank=True, related_name="emailing_to_be_received")
    sent_to = models.ManyToManyField(Contact, blank=True, related_name="emailing_received")
    opened_emails = models.ManyToManyField(Contact, blank=True, related_name="emailing_opened")
    status = models.IntegerField(default=STATUS_EDITING, choices=STATUS_CHOICES)

    hard_bounce = models.ManyToManyField(Contact, blank=True, related_name="emailing_hard_bounce")
    soft_bounce = models.ManyToManyField(Contact, blank=True, related_name="emailing_soft_bounce")
    spam = models.ManyToManyField(Contact, blank=True, related_name="emailing_spam")
    unsub = models.ManyToManyField(Contact, blank=True, related_name="emailing_unsub")
    rejected = models.ManyToManyField(Contact, blank=True, related_name="emailing_rejected")

    scheduling_dt = models.DateTimeField(_("scheduling date"), blank=True, default=None, null=True)
    sending_dt = models.DateTimeField(_("sending date"), blank=True, default=None, null=True)

    favorites = GenericRelation(Favorite)
    lang = models.CharField(_("language"), max_length=5, default="", blank=True)
    from_email = models.CharField(max_length=100, blank=True, default="", verbose_name=_("From email"))

    def __str__(self):
        return "{0} - {1}".format(self.newsletter.subject, self.get_info(extra=True))

    class Meta:
        verbose_name = _('emailing')
        verbose_name_plural = _('emailings')

    def get_info(self, extra=False):
        """returns info about the status in order to diplsay it on emailing page"""
        text = self.get_status_display()
        if self.status == Emailing.STATUS_EDITING:
            text = _("{0} - {1}").format(text, DateFormat(self.created).format(" d F Y H:i"))
            if extra:
                text += _(" - {0} recipients").format(self.send_to.count())
        if self.status == Emailing.STATUS_SCHEDULED:
            text = _("{0} - {1}").format(text, DateFormat(self.scheduling_dt).format(" d F Y H:i"))
            if extra:
                total = self.send_to.count() + self.sent_to.count()
                text += _(" - {0}/{1} sent").format(self.sent_to.count(), total)
        elif self.status == Emailing.STATUS_SENT:
            text = _("{0} - {1}").format(text, DateFormat(self.sending_dt).format(" d F Y H:i"))
            if extra:
                text += _(" - {0} sent").format(self.sent_to.count())

        return text
        
    def next_action(self):
        """what to to next"""
        action = ""
        if self.status == Emailing.STATUS_EDITING:
            action = '''
                <a class="colorbox-form action-button" href="{1}"><span class="glyphicon glyphicon-{2}"></span> {0}</a>
                '''.format(
                    ugettext('Send'),
                    reverse("emailing_confirm_send_mail", args=[self.id]),
                    'ok-circle'
                )
        if self.status == Emailing.STATUS_SCHEDULED:
            action = '''
                <a class="colorbox-form action-button" href="{1}"><span class="glyphicon glyphicon-{2}"></span> {0}</a>
                '''.format(
                    ugettext('Cancel'),
                    reverse("emailing_cancel_send_mail", args=[self.id]),
                    'remove-circle'
                )

        return mark_safe(action)

    def get_domain_url_prefix(self):
        """domain url prefix"""
        if self.subscription_type.site:
            domain_protocol = getattr(settings, "BALAFON_DOMAIN_PROTOCOL", "http://")
            return domain_protocol + self.subscription_type.site.domain
        return ""
    
    def get_contacts(self):
        """full list of contacts"""
        return list(self.send_to.all()) + list(self.sent_to.all())
        
    def save(self, *args, **kwargs):
        """save"""
        if self.lang and self.lang not in [value for (value, label) in get_language_choices()]:
            self.lang = ''
        if self.status == Emailing.STATUS_SENT and self.sending_dt is None:
            self.sending_dt = datetime.now()
        return super(Emailing, self).save(*args, **kwargs)


@python_2_unicode_compatible
class MagicLink(models.Model):
    """A tracking link"""

    class Meta:
        verbose_name = _('Magic link')
        verbose_name_plural = _('Magic links')

    emailing = models.ForeignKey(Emailing)
    url = models.URLField(max_length=500)
    visitors = models.ManyToManyField(Contact, blank=True)
    uuid = models.CharField(max_length=100, blank=True, default='', db_index=True)
    
    def __str__(self):
        return self.url
    
    def save(self, *args, **kwargs):
        """save"""
        super(MagicLink, self).save(*args, **kwargs)
        if not self.uuid:
            name = '{0}-magic-link-{1}-{2}'.format(settings.SECRET_KEY, self.id, self.url)
            ascii_name = unicodedata.normalize('NFKD', name).encode("ascii", 'ignore')
            self.uuid = uuid.uuid5(uuid.NAMESPACE_URL, '{0}'.format(ascii_name))
            return super(MagicLink, self).save()


def force_message_in_favorites(sender, instance, signal, created, **kwargs):
    """force an action to be in user favorites"""
    action = instance
    if created and action.type and action.type.name == ugettext("Message"):
        for user_pref in UserPreferences.objects.filter(message_in_favorites=True):
            content_type = ContentType.objects.get_for_model(action.__class__)
            Favorite.objects.get_or_create(
                user=user_pref.user,
                content_type=content_type,
                object_id=action.id
            )
            
signals.post_save.connect(force_message_in_favorites, sender=Action)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-09-28 13:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0016_auto_20180725_0905'),
    ]

    operations = [
        migrations.AlterField(
            model_name='saleitem',
            name='pre_tax_price',
            field=models.DecimalField(decimal_places=4, max_digits=11, verbose_name='pre-tax price'),
        ),
        migrations.AlterField(
            model_name='storeitem',
            name='pre_tax_price',
            field=models.DecimalField(decimal_places=4, max_digits=11, verbose_name='pre-tax price'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('Crm', '0011_auto_20160520_1734'),
    ]

    operations = [
        migrations.AddField(
            model_name='action',
            name='created_by',
            field=models.ForeignKey(related_name='+', default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='created by'),
        ),
        migrations.AddField(
            model_name='contact',
            name='created_by',
            field=models.ForeignKey(related_name='+', default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='created by'),
        ),
        migrations.AddField(
            model_name='entity',
            name='created_by',
            field=models.ForeignKey(related_name='+', default=None, blank=True, to=settings.AUTH_USER_MODEL, null=True, verbose_name='created by'),
        ),
    ]



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Store import models
from balafon.Store.admin import StockThresholdFilter, StoreItemAdmin


class StockThresholdListFilterTest(TestCase):
    """It should filter items according to threshold level"""

    def test_below_threshold(self):
        """It should display items below threshold"""
        item1 = mommy.make(models.StoreItem, stock_count=20, stock_threshold=30)
        mommy.make(models.StoreItem, stock_count=40, stock_threshold=30)
        item3 = mommy.make(models.StoreItem, stock_count=0, stock_threshold=30)
        mommy.make(models.StoreItem, stock_count=40, stock_threshold=None)
        mommy.make(models.StoreItem, stock_count=None, stock_threshold=None)

        list_filter = StockThresholdFilter(
            None,
            {StockThresholdFilter.parameter_name: StockThresholdFilter.THRESHOLD_ALERT},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual(
            sorted([item1, item3], key=lambda x: x.id),
            sorted(filtered_items.all(), key=lambda x: x.id)
        )

    def test_above_threshold(self):
        """It should filter display items above threshold"""
        mommy.make(models.StoreItem, stock_count=20, stock_threshold=30, name="Pomme")
        item2 = mommy.make(models.StoreItem, stock_count=40, stock_threshold=30, name="Poire")
        mommy.make(models.StoreItem, stock_count=0, stock_threshold=30, name="Banane")
        mommy.make(models.StoreItem, stock_count=40, stock_threshold=None, name="Raisin")
        mommy.make(models.StoreItem, stock_count=None, stock_threshold=None, name="Kiwi")

        list_filter = StockThresholdFilter(
            None,
            {StockThresholdFilter.parameter_name: StockThresholdFilter.THRESHOLD_OK},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual([item2], list(filtered_items.all()))

    def test_no_threshold(self):
        """It should filter items without threshold"""
        mommy.make(models.StoreItem, stock_count=20, stock_threshold=30)
        mommy.make(models.StoreItem, stock_count=40, stock_threshold=30)
        mommy.make(models.StoreItem, stock_count=0, stock_threshold=30)
        item4 = mommy.make(models.StoreItem, stock_count=40, stock_threshold=None)
        item5 = mommy.make(models.StoreItem, stock_count=None, stock_threshold=None)

        list_filter = StockThresholdFilter(
            None,
            {StockThresholdFilter.parameter_name: StockThresholdFilter.THRESHOLD_NONE},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual(
            sorted([item4, item5], key=lambda x: x.id),
            sorted(filtered_items.all(), key=lambda x: x.id)
        )

    def test_view_all(self):
        """It should return all"""
        item1 = mommy.make(models.StoreItem, stock_count=20, stock_threshold=30)
        item2 = mommy.make(models.StoreItem, stock_count=40, stock_threshold=30)
        item3 = mommy.make(models.StoreItem, stock_count=0, stock_threshold=30)
        item4 = mommy.make(models.StoreItem, stock_count=40, stock_threshold=None)
        item5 = mommy.make(models.StoreItem, stock_count=None, stock_threshold=None)

        list_filter = StockThresholdFilter(
            None,
            {},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual(
            sorted([item1, item2, item3, item4, item5], key=lambda x: x.id),
            sorted(filtered_items.all(), key=lambda x: x.id)
        )

    def test_warning_threshold_empty(self):
        """It should return empty list"""
        list_filter = StockThresholdFilter(
            None,
            {StockThresholdFilter.parameter_name: StockThresholdFilter.THRESHOLD_ALERT},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual(0, filtered_items.count())

    def test_ok_threshold_empty(self):
        """It should return empty list"""
        list_filter = StockThresholdFilter(
            None,
            {StockThresholdFilter.parameter_name: StockThresholdFilter.THRESHOLD_OK},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual(0, filtered_items.count())

    def test_no_threshold_empty(self):
        """It should return empty list"""
        list_filter = StockThresholdFilter(
            None,
            {StockThresholdFilter.parameter_name: StockThresholdFilter.THRESHOLD_NONE},
            models.StoreItem,
            StoreItemAdmin
        )
        filtered_items = list_filter.queryset(None, models.StoreItem.objects.all())
        self.assertEqual(0, filtered_items.count())




# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import template

from balafon.utils import logger
from django.db.models import Q
from balafon.Search.utils import get_date_bounds


register = template.Library()


@register.filter
def get_actions(contact, search_data):
    try:
        action_type = None
        planned_date = None
        done_date = None
        for search_block in search_data.values():
            for search_filter in search_block:
                if not action_type:
                    action_type = search_filter.get('action_type', None)
                if not planned_date:
                    planned_date = search_filter.get('action_by_planned_date', None)
                if not done_date:
                    done_date = search_filter.get('action_by_done_date', None)
        
        lookup_filter = []
        if action_type:
            lookup_filter += [Q(type=action_type)]
        if planned_date:
            dt_from, dt_to = get_date_bounds(planned_date)
            lookup_filter += [Q(planned_date__gte=dt_from) & Q(planned_date__lte=dt_to)]
        if done_date:
            dt_from, dt_to = get_date_bounds(done_date)
            lookup_filter += [Q(done_date__gte=dt_from) & Q(done_date__lte=dt_to)]
        if lookup_filter:
            return contact.action_set.filter(*lookup_filter)
        return contact.action_set.all()
    except:
        logger.exception("balafon_pdf.get_actions")
        raise





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0007_saleitem_no_quantity'),
    ]

    operations = [
        migrations.AddField(
            model_name='saleitem',
            name='is_discount',
            field=models.BooleanField(default=False, help_text='added after total on the bill', verbose_name='is discount'),
        ),
        migrations.AlterField(
            model_name='saleitem',
            name='no_quantity',
            field=models.BooleanField(default=False, help_text='quantity and unit price are not shown on bill', verbose_name='no quantity'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType
from django import template

from balafon.Users.forms import UpdateFavoriteForm
from balafon.Users.models import Favorite

register = template.Library()


@register.inclusion_tag('Users/_favorite_item.html', takes_context=True)
def favorite_item(context, object):
    content_type = ContentType.objects.get_for_model(object.__class__)
    user = context['request'].user
    if user and user.is_authenticated():
        is_in_favorite = Favorite.objects.filter(
            user=context['request'].user,
            content_type=content_type,
            object_id=object.id
        ).count()
    else:
        is_in_favorite = 0

    return {
        'status': is_in_favorite,
        'post_url': reverse('users_toggle_favorite'),
        'form': UpdateFavoriteForm(instance=object),
    }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SaleAnalysisCode',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'Delivery point',
                'verbose_name_plural': 'Delivery points',
            },
        ),
        migrations.AlterField(
            model_name='sale',
            name='action',
            field=models.OneToOneField(to='Crm.Action', verbose_name='action'),
        ),
        migrations.AddField(
            model_name='sale',
            name='analysis_code',
            field=models.ForeignKey(default=None, blank=True, to='Store.SaleAnalysisCode', null=True, verbose_name='analysis code'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.mail import EmailMessage
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render
from django.template.loader import get_template
from django.utils.translation import ugettext as _

from registration import get_version as get_registration_version
try:
    from registration.backends.default.views import RegistrationView, ActivationView
except ImportError:
    from registration.backends.model_activation.views import RegistrationView, ActivationView

from balafon.Crm.models import Action, ActionType
from balafon.Profile.forms import MessageForm
from balafon.Profile.models import ContactProfile
from balafon.Profile.settings import is_registration_enabled
from balafon.Profile.utils import create_profile_contact, notify_registration
from balafon.settings import get_profile_form, get_registration_form
from balafon.utils import now_rounded


@login_required 
def edit_profile(request):
    try:
        profile = request.user.contactprofile
    except ContactProfile.DoesNotExist:
        raise Http404

    profile_form_class = get_profile_form()
    
    if request.method == "POST":
        form = profile_form_class(request.POST, request.FILES, instance=profile.contact)
        if form.is_valid():
            #save contact
            form.save()
            messages.add_message(request, messages.SUCCESS, _("Your profile has been updated."))
            return HttpResponseRedirect(reverse('homepage'))
    else:
        form = profile_form_class(instance=profile.contact)
        
    return render(
        request,
        'Profile/edit_profile.html',
        {
            'contact': profile.contact,
            'form': form,
        }
    )


@login_required 
def post_message(request):
    try:
        profile = request.user.contactprofile
    except ContactProfile.DoesNotExist:
        raise Http404

    if not profile.contact:
        raise Http404
    
    if request.method == "POST":
        
        form = MessageForm(request.POST)
        if form.is_valid():
            
            message = form.cleaned_data['message']
            
            # send message by email
            notification_email = getattr(settings, 'BALAFON_NOTIFICATION_EMAIL', '')
            if notification_email:
                from_email = getattr(settings, 'DEFAULT_FROM_EMAIL')
                
                data = {
                    'contact': profile.contact,
                    'message': message,
                    'site': settings.COOP_CMS_SITE_PREFIX,
                }
                template_ = get_template('Emailing/subscribe_notification_email.txt')
                content = template_.render(data)
                email = EmailMessage(
                    _("Message from web site"), content, from_email,
                    [notification_email], headers = {'Reply-To': profile.contact.email})
                try:
                    email.send()
                    messages.add_message(request, messages.SUCCESS, _("The message have been sent"))
                except Exception:
                    messages.add_message(request, messages.ERROR, _("The message couldn't be send."))
                    
            # add an action
            message_action, _is_new = ActionType.objects.get_or_create(name=_('Message'))
            Action.objects.create(
                subject=_("New message on web site"), planned_date=now_rounded(),
                type=message_action, detail=message, contact=profile.contact, display_on_board=True
            )
        
            return HttpResponseRedirect(reverse('homepage'))
    else:
        form = MessageForm()
        
    return render(
        request,
        'Profile/post_message.html',
        {
            'contact': profile.contact,
            'form': form,
        }
    )


class AcceptNewsletterRegistrationView(RegistrationView):
    
    def get_form_class(self, request=None):
        return get_registration_form()

    def dispatch(self, request, *args, **kwargs):
        """make possible to disable this view from settings"""
        if not is_registration_enabled():
            raise Http404
        return super(AcceptNewsletterRegistrationView, self).dispatch(request, *args, **kwargs)
    
    def register(self, request_or_form, **kwargs):

        if get_registration_version() >= '2.0.0':
            data = request_or_form.cleaned_data
        else:
            data = kwargs
            kwargs["username"] = kwargs["email"][:30]

        user = super(AcceptNewsletterRegistrationView, self).register(request_or_form, **kwargs)

        user.first_name = data.get('firstname', "")
        user.last_name = data.get('lastname', "")

        user.contactprofile.firstname = data.get('firstname', "")
        user.contactprofile.lastname = data.get('lastname', "")

        user.contactprofile.entity_type = data.get('entity_type', None)
        user.contactprofile.entity_name = data.get('entity', "")

        user.contactprofile.phone = data.get('phone', "")
        user.contactprofile.mobile = data.get('mobile', "")

        user.contactprofile.city = data.get('city', None)
        user.contactprofile.zip_code = data.get('zip_code', None)
        user.contactprofile.gender = data.get('gender', 0) or 0

        user.contactprofile.address = data.get('address', "")
        user.contactprofile.address2 = data.get('address2', "")
        user.contactprofile.address3 = data.get('address3', "")
        user.contactprofile.cedex = data.get('cedex', "")
        user.contactprofile.birth_date = data.get('birth_date', None)

        user.contactprofile.groups_ids = data.get('groups_ids', '')

        user.save()
        user.contactprofile.save()

        subscription_types = data.get('subscription_types', None)

        user.contactprofile.subscriptions_ids = ",".join([str(s.id) for s in subscription_types])
        user.contactprofile.save()

        return user


class AcceptNewsletterActivationView(ActivationView):

    def activate(self, request, *args, **kwargs):
        activated_user = super(AcceptNewsletterActivationView, self).activate(request, *args, **kwargs)
        # The account has been activated: We can create the corresponding contact in Balafon
        if activated_user:
            profile = create_profile_contact(activated_user)
            notify_registration(profile)
        return activated_user



# -*- coding: utf-8 -*-
"""test we can search contacts by action"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy


from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class ActionForContactsTest(BaseTestCase):
    """create actions from results"""

    def test_view_create_action_for_contacts(self):
        """test GET on url"""
        url = reverse('search_create_action_for_contacts')
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)

    def test_view_form(self):
        """test view crete action form"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        url = reverse('search_create_action_for_contacts')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        id_contacts = soup.select('input#id_contacts')
        self.assertEqual(1, len(id_contacts))

        self.assertEqual(
            sorted([int(x) for x in id_contacts[0]['value'].split(';')]),
            sorted([contact1.id, contact2.id])
        )

    def test_view_form_not_logged(self, ):
        """test view form is not logged"""
        self.client.logout()

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        entity2 = mommy.make(models.Entity, name="Other corp")

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        url = reverse('search_create_action_for_contacts')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)
        # login url without lang prefix
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_post_create_actions_for_contacts(self):
        """test create actions for contact"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact
        entity3 = mommy.make(models.Entity, name="Big corp")
        contact3 = entity3.default_contact

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in (contact1.id, contact2.id)]),
            'date': '',
            'time': '',
            'type': '',
            'subject': 'test',
            'in_charge': '',
            'detail': '',
            'planned_date': '',
            'opportunity': '',
            'create_actions': '',
        }

        url = reverse('search_create_action_for_contacts')
        response = self.client.post(url, data)

        self.assertEqual(200, response.status_code)
        self.assertContains(response, '<script>$.colorbox.close();')

        self.assertEqual(models.Action.objects.count(), 2)

        self.assertEqual(models.Action.objects.filter(contacts=contact1).count(), 1)
        self.assertEqual(models.Action.objects.filter(contacts=contact2).count(), 1)
        self.assertEqual(models.Action.objects.filter(contacts=contact3).count(), 0)

        action1 = models.Action.objects.filter(contacts=contact1)[0]
        action2 = models.Action.objects.filter(contacts=contact2)[0]

        self.assertNotEqual(action1.id, action2.id)
        self.assertEqual(action1.contacts.count(), 1)
        self.assertEqual(action2.contacts.count(), 1)
        self.assertEqual(action1.subject, data['subject'])
        self.assertEqual(action2.subject, data['subject'])

    def test_post_create_actions_anonymous(self):
        """test can't create actions for contact if not logged"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact

        self.client.logout()

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in (contact1.id, contact2.id)]),
            'date': '',
            'time': '',
            'type': '',
            'subject': 'test',
            'in_charge': '',
            'detail': '',
            'planned_date': '',
            'opportunity': '',
            'create_actions': '',
        }

        url = reverse('search_create_action_for_contacts')
        response = self.client.post(url, data)

        self.assertEqual(302, response.status_code)
        #login url without lang prefix
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

        self.assertEqual(models.Action.objects.count(), 0)

    def test_post_create_actions_not_staff(self):
        """test can't create actions for contact if not in staff"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact

        self.user.is_staff = False
        self.user.save()

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in (contact1.id, contact2.id)]),
            'date': '',
            'time': '',
            'type': '',
            'subject': 'test',
            'in_charge': '',
            'detail': '',
            'planned_date': '',
            'opportunity': '',
            'create_actions': '',
        }

        url = reverse('search_create_action_for_contacts')
        response = self.client.post(url, data)

        self.assertEqual(302, response.status_code)
        # login url without lang prefix
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

        self.assertEqual(models.Action.objects.count(), 0)

    def test_post_create_actions_default_status(self):
        """test create actions for contact"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact
        entity3 = mommy.make(models.Entity, name="Big corp")
        contact3 = entity3.default_contact

        action_type = mommy.make(models.ActionType)
        action_status1 = mommy.make(models.ActionStatus)
        action_status2 = mommy.make(models.ActionStatus)
        action_type.allowed_status.add(action_status1)
        action_type.allowed_status.add(action_status2)
        action_type.default_status = action_status2
        action_type.save()

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in (contact1.id, contact2.id)]),
            'date': '',
            'time': '',
            'type': action_type.id,
            'subject': 'test',
            'in_charge': '',
            'detail': '',
            'planned_date': '',
            'opportunity': '',
            'create_actions': '',
        }

        url = reverse('search_create_action_for_contacts')
        response = self.client.post(url, data)

        self.assertEqual(200, response.status_code)
        self.assertContains(response, '<script>$.colorbox.close();')

        self.assertEqual(models.Action.objects.count(), 2)

        self.assertEqual(models.Action.objects.filter(contacts=contact1).count(), 1)
        self.assertEqual(models.Action.objects.filter(contacts=contact2).count(), 1)
        self.assertEqual(models.Action.objects.filter(contacts=contact3).count(), 0)

        action1 = models.Action.objects.filter(contacts=contact1)[0]
        action2 = models.Action.objects.filter(contacts=contact2)[0]

        self.assertNotEqual(action1.id, action2.id)

        for action in (action1, action2):
            self.assertEqual(action.contacts.count(), 1)
            self.assertEqual(action.subject, data['subject'])
            self.assertEqual(action.type, action_type)
            self.assertEqual(action.status, action_status2)

    def test_post_create_actions_opportunity(self):
        """test create actions for contact"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact
        entity3 = mommy.make(models.Entity, name="Big corp")
        contact3 = entity3.default_contact

        opportunity = mommy.make(models.Opportunity)

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in (contact1.id, contact2.id)]),
            'date': '',
            'time': '',
            'type': '',
            'subject': 'test',
            'in_charge': '',
            'detail': '',
            'planned_date': '',
            'opportunity': opportunity.id,
            'create_actions': '',
        }

        url = reverse('search_create_action_for_contacts')
        response = self.client.post(url, data)

        self.assertEqual(200, response.status_code)
        self.assertContains(response, '<script>$.colorbox.close();')

        self.assertEqual(models.Action.objects.count(), 2)

        self.assertEqual(models.Action.objects.filter(contacts=contact1).count(), 1)
        self.assertEqual(models.Action.objects.filter(contacts=contact2).count(), 1)
        self.assertEqual(models.Action.objects.filter(contacts=contact3).count(), 0)

        action1 = models.Action.objects.filter(contacts=contact1)[0]
        action2 = models.Action.objects.filter(contacts=contact2)[0]

        self.assertNotEqual(action1.id, action2.id)

        for action in (action1, action2):
            self.assertEqual(action.contacts.count(), 1)
            self.assertEqual(action.subject, data['subject'])
            self.assertEqual(action.opportunity, opportunity)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0016_auto_20170130_0955'),
    ]

    operations = [
        migrations.AddField(
            model_name='zone',
            name='groups',
            field=models.ManyToManyField(related_name='children_groups_set', verbose_name='parent groups', to='Crm.Zone', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0017_zone_groups'),
    ]

    operations = [
        migrations.AddField(
            model_name='action',
            name='status2',
            field=models.ForeignKey(related_name='status2_set', default=None, blank=True, to='Crm.ActionStatus', null=True),
        ),
        migrations.AddField(
            model_name='actiontype',
            name='allowed_status2',
            field=models.ManyToManyField(default=None, help_text='Action of this type allow the given status', related_name='type_status2_set', to='Crm.ActionStatus', blank=True),
        ),
        migrations.AddField(
            model_name='actiontype',
            name='default_status2',
            field=models.ForeignKey(related_name='type_default_status2_set', default=None, blank=True, to='Crm.ActionStatus', help_text='Default status for actions of this type', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0011_auto_20171006_1528'),
    ]

    operations = [
        migrations.AddField(
            model_name='storeitem',
            name='description',
            field=models.TextField(default=b'', verbose_name='description', blank=True),
        ),
        migrations.AddField(
            model_name='storeitem',
            name='image',
            field=models.ImageField(default=None, upload_to=b'storeitems', null=True, verbose_name='image', blank=True),
        ),
        migrations.AddField(
            model_name='storeitemcategory',
            name='default_image',
            field=models.ImageField(default=None, upload_to=b'storeitemcats', null=True, verbose_name='image', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages

VERSION = __import__('balafon').__version__

setup(
    name='balafon',
    version=VERSION,
    description='Contacts management',
    packages=find_packages(),
    include_package_data=True,
    author='Luc Jean',
    author_email='ljean@apidev.fr',
    license='CeCILL 2.1',
    zip_safe=False,
    install_requires=[
        'django >= 1.11.15, <2.0',
        'djangorestframework',
        'django-extensions',
        'apidev-coop_cms>=1.3.3',
        'beautifulsoup4',
        'django-wkhtmltopdf',
        'xlwt',
        'xlrd',
        'django-recaptcha2',
        'django-registration<3.0',
        'model_mommy',
        'django-cors-headers',
    ],
    long_description=open('README.rst').read(),
    url='https://github.com/ljean/balafon',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Natural Language :: English',
        'Natural Language :: French',
        'Topic :: Office/Business',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-04-09 13:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Store', '0012_auto_20180104_1728'),
    ]

    operations = [
        migrations.AlterField(
            model_name='certificate',
            name='logo',
            field=models.ImageField(blank=True, default='', upload_to='certificates'),
        ),
        migrations.AlterField(
            model_name='discount',
            name='short_name',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='short name'),
        ),
        migrations.AlterField(
            model_name='priceclass',
            name='description',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='pricepolicy',
            name='parameters',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='parameters'),
        ),
        migrations.AlterField(
            model_name='pricepolicy',
            name='policy',
            field=models.CharField(choices=[('from_category', 'Inherit from category'), ('multiply_purchase_by_ratio', 'Multiply purchase price by ratio')], max_length=100, verbose_name='policy'),
        ),
        migrations.AlterField(
            model_name='saleitem',
            name='text',
            field=models.TextField(blank=True, default='', max_length=3000, verbose_name='Text'),
        ),
        migrations.AlterField(
            model_name='storeitem',
            name='description',
            field=models.TextField(blank=True, default='', verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='storeitem',
            name='image',
            field=models.ImageField(blank=True, default=None, null=True, upload_to='storeitems', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='storeitem',
            name='origin',
            field=models.CharField(blank=True, default='', max_length=50, verbose_name='Origine'),
        ),
        migrations.AlterField(
            model_name='storeitem',
            name='reference',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='reference'),
        ),
        migrations.AlterField(
            model_name='storeitemcategory',
            name='default_image',
            field=models.ImageField(blank=True, default=None, null=True, upload_to='storeitemcats', verbose_name='image'),
        ),
        migrations.AlterField(
            model_name='storeitemcategory',
            name='icon',
            field=models.CharField(blank=True, default='', max_length=20),
        ),
        migrations.AlterField(
            model_name='storeitemimport',
            name='data',
            field=models.FileField(help_text='An Excel file (*.xls) with the data', upload_to='store_item_imports', verbose_name='Import file'),
        ),
        migrations.AlterField(
            model_name='storeitemimport',
            name='default_brand',
            field=models.CharField(blank=True, default='', help_text='If defined, it will be used if no brand is given', max_length=50, verbose_name='default Brand'),
        ),
        migrations.AlterField(
            model_name='storeitemimport',
            name='error_message',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='error message'),
        ),
        migrations.AlterField(
            model_name='storeitemimport',
            name='fields',
            field=models.CharField(default='name,brand,reference,category,purchase_price,vat_rate', help_text="Fields to import in order: if the attribute doesn't exist, create a custom property", max_length=300, verbose_name='fields'),
        ),
        migrations.AlterField(
            model_name='storeitemimport',
            name='import_error',
            field=models.TextField(blank=True, default='', verbose_name='import error'),
        ),
        migrations.AlterField(
            model_name='storeitemproperty',
            name='label',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='label'),
        ),
        migrations.AlterField(
            model_name='storeitempropertyvalue',
            name='value',
            field=models.CharField(blank=True, default='', max_length=100, verbose_name='value'),
        ),
        migrations.AlterField(
            model_name='storeitemtag',
            name='icon',
            field=models.CharField(blank=True, default='', max_length=20),
        ),
        migrations.AlterField(
            model_name='storemanagementactiontype',
            name='references_text',
            field=models.TextField(blank=True, default='', help_text='this text will be added at the bottom of the commercial document', verbose_name='references text'),
        ),
        migrations.AlterField(
            model_name='storemanagementactiontype',
            name='template_name',
            field=models.CharField(blank=True, default='', help_text='Set the name of a custom template for commercial document', max_length=100, verbose_name='template name'),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext as _

from coop_bar.utils import make_link


def can_view(perm, object_names):
    def inner_decorator(func):
        def wrapper(request, context):
            for object_name in object_names:
                obj = context.get(object_name)
                if obj and request and request.user.has_perm(perm+"_"+object_name, obj):
                    yes_we_can = func(request, context)
                    if yes_we_can:
                        return yes_we_can
            return
        return wrapper
    return inner_decorator

can_view_object = can_view('can_view', ['object'])


@can_view_object
def doc_to_pdf(request, context):
    obj = context.get("object", None)
    if obj:
        if not context.get('edit_mode') and hasattr(obj, 'get_pdf_url'):
            return make_link(obj.get_pdf_url(), _('Pdf'), 'fugue/document-pdf.png',
                classes=['icon', 'alert_on_click'])


def load_commands(coop_bar):
    
    coop_bar.register([
        [doc_to_pdf],
    ])



# -*- coding: utf-8 -*-
"""templatetags library"""

from __future__ import unicode_literals

from datetime import date, datetime

from django import template
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils.safestring import mark_safe

from balafon.Crm.models import ActionStatusTrack
from balafon.utils import logger

register = template.Library()


@register.filter
def seq_to_br(seq):
    """return sequence items separated by br html tags"""
    if seq:
        return mark_safe("<br>".join(['{0}'.format(x) for x in seq]))
    return mark_safe("&nbsp;")


@register.filter
def seq_to_dash(seq):
    """return sequence items separated by dashes"""
    if seq:
        return mark_safe(" - ".join(seq))
    return mark_safe("&nbsp;")


@register.filter
def is_today(date_time):
    """True if is today"""
    if date_time:
        date_val = date(date_time.year, date_time.month, date_time.day)
        return date_val == date.today()
    return False


@register.filter
def cut_null_hour(value):
    """remove the hour if 00:00"""
    try:
        date_and_hour = value.split(" ")
        date_, hour_ = ' '.join(date_and_hour[:-1]), date_and_hour[-1]
        if hour_ == "00:00":
            return date_
    except ValueError:
        pass
    return value


@register.filter
def get_action_date_label(action):
    """get label level (warning, danger) if date is raised"""
    if not action.planned_date:
        return "label-warning"
    if (not action.done) and action.planned_date < datetime.now():
        return "label-danger"
    return "label-not-yet"


@register.filter
def custom_field(instance, field_name):
    """access a custom field value"""
    return getattr(instance, 'custom_field_' + field_name)
    

class IncludeNode(template.Node):
    """
    try to load a template
    #Thanks http://djangosnippets.org/snippets/2058/
    """
    def __init__(self, template_name):
        self.template_name = template_name

    def render(self, context):
        try:
            # Loading the template and rendering it
            included_template = template.loader.get_template(self.template_name).render(context)
        except template.TemplateDoesNotExist:
            included_template = ''
        return included_template


@register.tag
def try_to_include(parser, token):
    """
    Usage: {% try_to_include "head.html" %}
    This will fail silently if the template doesn't exist. If it does, it will
    be rendered with the current context.
    """
    try:
        tag_name, template_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("{0} tag requires a single argument".format(token.contents.split()[0]))

    return IncludeNode(template_name[1:-1])


@register.filter
def split_lines(lines, nb=1):
    """split in lines"""
    try:
        return "\n".join(lines.splitlines()[:nb])
    except Exception:
        return ""


class MenuActionUrlNode(template.Node):
    """try to return a reverse without breaking app if invalid"""
    def __init__(self, view_name, action_id):
        self.view_name_var = view_name
        self.action_id_var = action_id

    def render(self, context):
        """return text"""
        try:
            obj, attr = self.view_name_var.split('.')
            view_name = getattr(context[obj], attr)

            obj, attr = self.action_id_var.split('.')
            action_id = getattr(context[obj], attr)

        except (ValueError, KeyError, AttributeError) as exc:
            logger.error('MenuActionUrl: {0}', exc)
            return ''

        try:
            action_id = int(action_id)
        except ValueError:
            logger.error("menu_action_url tag: action_id must be an integer")
            return ''

        try:
            return reverse(view_name, args=[action_id])
        except NoReverseMatch:
            logger.error(
                'MenuActionUrl: NoReverseMatch for reverse("{0}", args=[{1}])', self.view_name_var, self.action_id_var
            )
            return ''


@register.tag
def menu_action_url(parser, token):
    """return a reverse for action menu without breaking app if invalid """
    try:
        tag_name, view_name_var, action_id_var = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError("menu_action_url tag requires view_name and action_id")
    return MenuActionUrlNode(view_name_var, action_id_var)


@register.filter
def action_status_date(action, status):
    """return date"""
    queryset = ActionStatusTrack.objects.filter(action=action, status=status).order_by('-datetime')
    if queryset.count():
        return queryset[0].datetime



# -*- coding: utf-8 -*-
"""
coop_cms manage compatibilty with django and python versions
"""

import json
import sys

from django import VERSION
from django.conf import settings


if sys.version_info[0] < 3:
    # Python 2
    import urllib2

    def urlopen(url):
        return urllib2.urlopen(url).read()
else:
    # Python 3
    from urllib.request import urlopen



# -*- coding: utf-8 -*-
"""display actions in different planning views"""

from __future__ import unicode_literals

from datetime import datetime, date
import json

from django.contrib.auth.decorators import user_passes_test
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import Http404, HttpResponse
from django.views.generic import RedirectView
from django.views.generic.dates import MonthArchiveView, WeekArchiveView, DayArchiveView
from django.utils.decorators import method_decorator
from django.views.generic import ListView

from balafon.Crm import models
from balafon.Crm.utils import get_in_charge_users
from balafon.permissions import can_access
try:
    from balafon.Users.models import CustomMenu
    HAS_CUSTOM_MENU = True
except ImportError:
    HAS_CUSTOM_MENU = False


class ActionArchiveView(object):
    """view"""
    paginate_by = 50
    planning_type = ''

    @method_decorator(user_passes_test(can_access))
    def dispatch(self, *args, **kwargs):
        return super(ActionArchiveView, self).dispatch(*args, **kwargs)

    def _get_selection(self, filter_value):
        """filtering"""
        try:
            values = {}
            for item in filter_value.split(","):
                prefix, val = item[0], int(item[1:])
                if prefix in values:
                    values[prefix].append(val)
                else:
                    values[prefix] = [val]
            return dict(values)
        except ValueError:
            raise Http404

    def _get_queryset(self):
        """return actions displayed by this page. The date is managed by each Archive"""
        return models.Action.objects.all().order_by("planned_date", "priority")

    def get_queryset(self):
        """queryset ob objects"""
        values = self.request.GET.get("filter", None)
        queryset = self._get_queryset()
        if values and values != "null":
            values_dict = self._get_selection(values)

            selected_types = values_dict.get("t", [])
            if selected_types:
                if 0 in selected_types:
                    if len(selected_types) == 1:
                        # only : no types
                        queryset = queryset.filter(type__isnull=True)
                    else:
                        # combine no types and some types
                        queryset = queryset.filter(Q(type__isnull=True) | Q(type__in=selected_types))
                else:
                    # only some types
                    queryset = queryset.filter(type__in=selected_types)

            selected_status = values_dict.get("s", [])
            if selected_status:
                # only some status
                action_types = models.ActionType.objects.filter(id__in=selected_types)
                allowed_status = [status.id for status in self._get_allowed_status(action_types, selected_types)]
                selected_status = [status for status in selected_status if status in allowed_status]
                if selected_status:
                    queryset = queryset.filter(status__in=selected_status)

            selected_users = values_dict.get("u", [])
            if selected_users:
                queryset = queryset.filter(in_charge__in=selected_users)

            # Ordering
            actions_ordering = values_dict.get("o", [1])
            if actions_ordering == [0]:
                queryset = queryset.order_by("-planned_date", "-id")
            elif actions_ordering == [1]:
                queryset = queryset.order_by("planned_date", "id")

        return queryset

    def _get_allowed_status(self, action_types, selected_types):
        """list of allowed status"""
        action_status = []
        for action_type in action_types:
            if action_type.id in selected_types:
                setattr(action_type, 'selected', True)
                action_status.extend(action_type.allowed_status.all())
        # unique action status sorted by name
        return sorted(set(action_status), key=lambda status: status.name)

    def get_context_data(self, *args, **kwargs):
        """get context for template"""
        context = super(ActionArchiveView, self).get_context_data(*args, **kwargs)
        action_types = models.ActionType.objects.all().order_by('order_index', 'name')
        action_status = []
        in_charge = get_in_charge_users()
        values = self.request.GET.get("filter", None)
        ordering = "asc"
        if values and values != "null":
            context["filter"] = values
            values_dict = self._get_selection(values)

            selected_types = values_dict.get("t", [])
            for action_type in action_types:
                if action_type.id in selected_types:
                    setattr(action_type, 'selected', True)

            # list of unique action status sorted by name
            action_status = self._get_allowed_status(action_types, selected_types)

            if 0 in selected_types:
                context["no_type_selected"] = True

            selected_users = values_dict.get("u", [])
            for user in in_charge:
                if user.id in selected_users:
                    setattr(user, 'selected', True)

            selected_status = values_dict.get("s", [])
            for status in action_status:
                if status.id in selected_status:
                    setattr(status, 'selected', True)

            actions_ordering = values_dict.get("o", [1])
            if actions_ordering == [0]:
                ordering = "desc"
            elif actions_ordering == [1]:
                ordering = "asc"

        if HAS_CUSTOM_MENU:
            context['planning_custom_menus'] = [
                menu for menu in CustomMenu.objects.filter(position=CustomMenu.POSITION_PLANNING) if menu.get_children()
            ]

        context["action_types"] = action_types
        context["in_charge"] = in_charge
        context["action_status"] = action_status
        context["planning_type"] = self.planning_type
        context["ordering"] = ordering

        return context

    def get_dated_queryset(self, **lookup_kwargs):
        """queryset"""
        date_field = self.get_date_field()

        since = lookup_kwargs['%s__gte' % date_field]
        until = lookup_kwargs['%s__lt' % date_field]

        lookup1 = Q(end_datetime__isnull=True) & Q(planned_date__gte=since) & Q(planned_date__lt=until)
        lookup2 = Q(end_datetime__isnull=False) & Q(planned_date__lt=until) & Q(end_datetime__gte=since)

        queryset = self.get_queryset()
        return queryset.filter(lookup1 | lookup2)

    def get(self, *args, **kwargs):
        """http get"""
        self.request.session["redirect_url"] = self.request.get_full_path()
        return super(ActionArchiveView, self).get(*args, **kwargs)


class ActionMonthArchiveView(ActionArchiveView, MonthArchiveView):
    """view"""
    date_field = "planned_date"
    month_format = '%m'
    allow_future = True
    allow_empty = True
    planning_type = 'month'


class ActionWeekArchiveView(ActionArchiveView, WeekArchiveView):
    """view"""
    date_field = "planned_date"
    week_format = "%W"
    allow_future = True
    allow_empty = True
    planning_type = 'week'


class ActionDayArchiveView(ActionArchiveView, DayArchiveView):
    """view"""
    date_field = "planned_date"
    allow_future = True
    allow_empty = True
    month_format = '%m'
    planning_type = 'day'


class NotPlannedActionArchiveView(ActionArchiveView, ListView):
    """view"""
    queryset = models.Action.objects.filter(planned_date=None).order_by("priority")
    template_name = "Crm/action_archive_not_planned.html"

    def _get_queryset(self):
        """return actions displayed by this page"""
        return models.Action.objects.filter(planned_date=None).order_by("priority")


class TodayActionsView(RedirectView):
    """Redirect to today action"""
    permanent = False
    query_string = True

    def get_redirect_url(self, *args, **kwargs):
        now = datetime.now()
        self.url = reverse('crm_actions_of_day', args=[now.year, now.month, now.day])
        return super(TodayActionsView, self).get_redirect_url(*args, **kwargs)


class ThisWeekActionsView(RedirectView):
    """Redirect this week actions action"""
    permanent = False
    query_string = True

    def get_redirect_url(self, *args, **kwargs):
        now = datetime.now()
        self.url = reverse('crm_actions_of_week', args=[now.year, now.strftime("%W")])
        return super(ThisWeekActionsView, self).get_redirect_url(*args, **kwargs)


class ThisMonthActionsView(RedirectView):
    """Redirect this month actions action"""
    permanent = False
    query_string = True

    def get_redirect_url(self, *args, **kwargs):
        now = datetime.now()
        self.url = reverse('crm_actions_of_month', args=[now.year, now.month])
        return super(ThisMonthActionsView, self).get_redirect_url(*args, **kwargs)


@user_passes_test(can_access)
def go_to_planning_date(request):
    """returns the url for displaying date"""

    if request.method != 'POST':
        raise Http404

    planning_type = request.POST.get('planning_type', '')
    planning_date = request.POST.get('planning_date', '')
    filters = request.POST.get('filters', '')

    try:
        planning_date = date(*[int(val) for val in planning_date.split('-')])
    except ValueError:
        raise Http404

    url = ''
    if planning_type == 'month':
        url = reverse('crm_actions_of_month', args=[planning_date.year, planning_date.month])

    elif planning_type == 'week':
        url = reverse('crm_actions_of_week', args=[planning_date.year, int(planning_date.strftime("%W"))])

    elif planning_type == 'day':
        url = reverse('crm_actions_of_day', args=[planning_date.year, planning_date.month, planning_date.day])

    if url:
        if filters:
            url += '?filter=' + filters
        data = json.dumps({'url': url})
        return HttpResponse(data, content_type='application/json')
    else:
        raise Http404



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

import json
import logging
import os.path
import shutil

from django.conf import settings
from django.test import TestCase as DjangoTestCase
from django.test.utils import override_settings

from coop_cms.utils import RequestManager


default_media_root = settings.MEDIA_ROOT


@override_settings(MEDIA_ROOT=os.path.join(default_media_root, '_unit_tests'))
class TestCase(DjangoTestCase):
    """Base class for tests"""

    def _clean_files(self):
        if default_media_root != settings.MEDIA_ROOT:
            try:
                shutil.rmtree(settings.MEDIA_ROOT)
            except OSError:
                pass
        else:
            raise Exception("Warning! wrong media root for unittesting")

    def setUp(self):
        """before each test"""
        RequestManager().clean()
        logging.disable(logging.CRITICAL)
        self._clean_files()

    def tearDown(self):
        """after each test"""
        logging.disable(logging.NOTSET)
        self._clean_files()


def response_as_json(response):
    return json.loads(response.content.decode('utf-8'))



# -*- coding: utf-8 -*-
"""settings"""

from __future__ import unicode_literals

from django.conf import settings
from django.views.generic import TemplateView
from django.utils.translation import ugettext as _

from coop_cms.settings import load_class
from wkhtmltopdf.views import PDFTemplateView


def get_allowed_homepages():
    """returns all Balafon pages which can be set as homepage"""
    return (
        # CRM
        'crm_view_entities_list',
        'crm_view_entity',
        'crm_view_contact',
        'crm_view_entity_actions',
        'crm_entity_actions',
        'crm_contact_actions',
        'crm_all_actions',
        'crm_edit_group',
        'crm_see_my_groups',
        'crm_view_opportunity',
        'crm_all_opportunities',
        'crm_new_contacts_import',
        'crm_actions_of_month',
        'crm_actions_of_week',
        'crm_actions_of_day',
        'crm_actions_not_planned',

        # Emailing
        'emailing_newsletter_list',
        'emailing_view',

        # Search
        'search',
        'search_group',
        'search_cities',

        # Users
        'users_favorites_list',
    )


def get_pdf_view_base_class():
    """returns base class for Pdf --> turn it to HTML for drone.IO"""
    if getattr(settings, 'CI_DRONE', False):
        return TemplateView
    else:
        return PDFTemplateView


def get_profile_form():
    """returns a form to be used for editing a user profile"""
    return load_class('BALAFON_PROFILE_FORM', 'balafon.Profile.forms.ProfileForm')


def get_registration_form():
    """returns a form to be used for editing a user profile"""
    return load_class('BALAFON_REGISTRATION_FORM', 'balafon.Profile.forms.UserRegistrationForm')


def has_entity_on_registration_form():
    """
    returns True if entity type and entity name are displayed on registration form (Profile)
    register as individual if not
    """
    return getattr(settings, 'BALAFON_ENTITY_ON_REGISTRATION_FORM', True)


def get_registration_accept_terms_of_use_link():
    """returns a link for the use of terms"""
    return getattr(settings, 'BALAFON_REGISTRATION_ACCEPT_USE_OF_TERMS_LINK', '')


def get_captcha_field():
    """return the captcha field to use"""

    default_field = 'snowpenguin.django.recaptcha2.fields.ReCaptchaField'
    default_widget = 'snowpenguin.django.recaptcha2.widgets.ReCaptchaWidget'

    captcha_field = load_class('BALAFON_CAPTCHA_FIELD', default_field)
    captcha_widget = load_class('BALAFON_CAPTCHA_WIDGET', default_widget)
    captcha_help_text = getattr(settings, 'BALAFON_CAPTCHA_HELP_TEXT', _(u"Make sure you are a human"))

    if captcha_widget:
        return captcha_field(widget=captcha_widget, help_text=captcha_help_text)
    else:
        return captcha_field(help_text=captcha_help_text)


def is_profile_installed():
    """returns True of balafon Profile is installed"""
    installed_apps = getattr(settings, 'INSTALLED_APPS', [])
    return 'balafon.Profile' in installed_apps



# -*- coding: utf-8 -*-
"""test we cans earch contacts y group"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from model_mommy import mommy

from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class AddToGroupActionTest(BaseTestCase):
    """Test the 'Add contact to group' action performed on search results"""

    def test_add_contact_to_group(self):
        """add a contact to group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group = mommy.make(models.Group, name="GROUP1")
        group.entities.add(entity1)
        group.entities.add(entity2)
        group.save()

        group2 = mommy.make(models.Group, name="GROUP2")
        self.assertEqual(group2.entities.count(), 0)
        self.assertEqual(group2.contacts.count(), 0)

        contacts = [contact1, contact2, contact3]
        url = reverse('search_add_contacts_to_group')
        data = {
            "gr0-_-group-_-0": group.id,
            'add_to_group': 'add_to_group',
            'groups': [group2.id],
            'on_contact': True,
            'contacts': ";".join([str(contact.id) for contact in contacts])
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertContains(
            response,
            '<script>$.colorbox.close(); window.location="{0}";</script>'.format(reverse('crm_board_panel'))
        )

        group2 = models.Group.objects.get(id=group2.id)

        for contact in contacts:
            self.assertTrue(contact in group2.contacts.all())

        for entity in [entity1, entity2]:
            self.assertFalse(entity in group2.entities.all())

    def test_add_entity_to_group(self):
        """add entity to group"""
        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCD", main_contact=True, has_left=False)
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKL", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", main_contact=True, has_left=False)

        group = mommy.make(models.Group, name="GROUP1")
        group.entities.add(entity1)
        group.entities.add(entity2)
        group.save()

        group2 = mommy.make(models.Group, name="GROUP2")
        self.assertEqual(group2.entities.count(), 0)
        self.assertEqual(group2.contacts.count(), 0)

        contacts = [contact1, contact2, contact3]
        url = reverse('search_add_contacts_to_group')
        data = {
            "gr0-_-group-_-0": group.id,
            'add_to_group': 'add_to_group',
            'groups': [group2.id],
            'on_contact': False,
            'contacts': ";".join([str(contact.id) for contact in contacts])
        }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertContains(
            response,
            '<script>$.colorbox.close(); window.location="{0}";</script>'.format(reverse('crm_board_panel'))
        )

        group2 = models.Group.objects.get(id=group2.id)

        for contact in contacts:
            self.assertFalse(contact in group2.contacts.all())

        for entity in [entity1, entity2]:
            self.assertTrue(entity in group2.entities.all())



# -*- coding: utf-8 -*-
"""
app configuration
"""

from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class BalafonAppConfig(AppConfig):
    name = 'balafon.Search'
    verbose_name = _("Balafon Search")



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os.path

from django.contrib.auth.models import User

from balafon.unit_tests import TestCase


class BaseTestCase(TestCase):
    """Base class for test cases"""

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.user = User.objects.create(username="toto")
        self.user.set_password("abc")
        self.user.save()
        self._login()

    def tearDown(self):
        super(BaseTestCase, self).tearDown()

    def _get_file(self, file_name='unittest1.txt'):
        full_name = os.path.normpath(os.path.dirname(__file__) + '/fixtures/' + file_name)
        return open(full_name, 'rb')

    def _login(self):
        self.client.login(username="toto", password="abc")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0013_mailtosettings'),
    ]

    operations = [
        migrations.AlterField(
            model_name='actionmenu',
            name='only_for_status',
            field=models.ManyToManyField(to='Crm.ActionStatus', verbose_name='Only for status', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
"""Crm is the main module"""

from __future__ import unicode_literals

from django.contrib.auth.decorators import user_passes_test
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import render, get_object_or_404

from colorbox.decorators import popup_redirect

from balafon.Crm import forms
from balafon.permissions import can_access


@user_passes_test(can_access)
@popup_redirect
def edit_custom_fields(request, model_name, instance_id):
    """view"""
    try:
        form_class = {
            'entity': forms.EntityCustomFieldForm,
            'contact': forms.ContactCustomFieldForm,
        }[model_name]
    except KeyError:
        raise Http404

    instance = get_object_or_404(form_class.model(), id=instance_id)

    if request.method == 'POST':
        form = form_class(instance, request.POST)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(instance.get_absolute_url())
    else:
        form = form_class(instance)

    return render(
        request,
        'Crm/edit_custom_fields.html',
        {'form': form, 'instance': instance, 'model_name': model_name},
    )
